/* Common definitions for all executor's parts.
 *
 * 2.10.89 Kenneth Oksanen
 *   First Version
 */


#ifndef INCL_DEFS
#define INCL_DEFS 1


/* Changing these implies more changes only in the lexer and in the output
 * routines.
 */

typedef int stacktype_t; 


/* Type of nodes in the symbol tree.
 */

typedef struct sym_tree_node_t {
  char *name;
  union {
    int address;
    stacktype_t value;
  } data;
  struct sym_tree_node_t *left, *right;
} sym_tree_node_t;


/* Instruction opcodes in the program.
 */

typedef enum {
  END_OF_PROGRAM = 0,
  PLUS, MINUS, MULT, DIV, MOD, PUSH, RVALUE, LVALUE, POP, SETQ,
  COPY, LABEL, GOTO, GOFALSE, GOTRUE, HALT, PRINT, 
  NUMERICAL_CONSTANT, SYMBOL_CONSTANT,
} opcode_t;


/* Type of an instruction.
 */

typedef struct {
  opcode_t opcode;
  union {
    stacktype_t imm_value;
    sym_tree_node_t *symbol;
  } arg;
} instruction_t;


#define YYSTYPE \
union { \
  stacktype_t number; \
  sym_tree_node_t *symbol; \
}


#endif /* INCL_DEFS */
