%{

/* Lex file for executor.c .
 *
 * 2.10.89 Kenneth Oksanen
 *   First Version
 *
 * 15.10.89 Kenneth Oksanen, Pekka Taipale
 *   Changed definitions of SYMBOLs.
 */

char *xstrdup(const char *const orig)
{
  char *new = (char *) malloc(strlen(orig) + 1); /* +1 for the last '\0' */
  if (new == NULL) {
    fprintf(stderr, "Memory exhausted\n");
    exit(0);
  }
  strcpy(new, orig);
  return new;
}

%}

%%

[ \t\n]				{ ; /* Pass 'em */ 	}
\+				{ return PLUS;		}
\-				{ return MINUS; 	}
\*				{ return MULT;		}
\/				{ return DIV;		}
[dD][iI][vV]			{ return DIV;		}
[mM][oO][dD]			{ return MOD;		}
[pP][uU][sS][hH] 		{ return PUSH;		}
[rR][vV][aA][lL][uU][eE] 	{ return RVALUE;	}
[lL][vV][aA][lL][uU][eE]	{ return LVALUE;	}
[pP][oO][pP]			{ return POP;		}
:=				{ return SETQ;		}
[cC][oO][pP][yY]		{ return COPY;		}
[lL][aA][bB][eE][lL]		{ return LABEL;		}
[gG][oO][tT][oO]		{ return GOTO;		}
[gG][oO][fF][aA][lL][sS][eE]	{ return GOFALSE;	}
[gG][oO][tT][rR][uU][eE]	{ return GOTRUE;	}
[hH][aA][lL][tT]		{ return HALT;		}
[pP][rR][iI][nN][tT]		{ return PRINT;	}
[-+]?[0-9]+			{ yylval.number = atoi(yytext);
				  return NUMERICAL_CONSTANT; }
[a-zA-Z_][a-zA-Z_0-9]*    { if((yylval.symbol = get_sym(yytext)) == NULL)
                              yylval.symbol = add_sym(xstrdup(yytext));
                            return SYMBOL_CONSTANT;     }

%%

yywrap()
{
  extern FILE *yyin;

  if(yyin != stdin)
    fclose(yyin);
  return 1;
}
