/* Stack definitions and routines.
 *
 * 3.10.89 Kenneth Oksanen
 *   Separated this module from the executor.c .
 */

#include <stdio.h>
#include "defs.h"

#define MAX_STACK_DEPTH 500


/* Each element in the stack contains a tag which tells its type. Thus
 * `executor' is a dynamically typed and safe language.
 */

static struct {
  enum { NUMBER, SYMBOL } type;
  union {
    stacktype_t number;
    sym_tree_node_t *symbol;
  } val;
} stack[MAX_STACK_DEPTH];

static int stack_p = 0;


stacktype_t pop(void)
{
  if (stack_p == 0) {
    fprintf(stderr, "Stack underflow.\n");
    exit(1);
  } else if (stack[stack_p-1].type != NUMBER) {
    fprintf(stderr, "Stack type mismatch.\n");
    exit(1);
  } else
    return stack[--stack_p].val.number;
}

void push(stacktype_t item)
{
  if (stack_p == MAX_STACK_DEPTH) {
    fprintf(stderr, "Stack overflow.\n");
    exit(1);
  } else {
    stack[stack_p].type = NUMBER;
    stack[stack_p++].val.number = item;
  }
}
    

/* Same for symbol-tree-entries...
 */

sym_tree_node_t *pop_sym(void)
{
  if (stack_p == 0) {
    fprintf(stderr, "Stack underflow.\n");
    exit(1);
  } else if(stack[stack_p-1].type != SYMBOL) {
    fprintf(stderr, "Stack type mismatch.\n");
    exit(1);
  } else
    return stack[--stack_p].val.symbol;
}

void push_sym(sym_tree_node_t *item)
{
  if (stack_p == MAX_STACK_DEPTH) {
    fprintf(stderr, "Stack overflow.\n");
    exit(1);
  } else {
    stack[stack_p].type = SYMBOL;
    stack[stack_p++].val.symbol = item;
  }
}
    
/* A dup-routine for the stack-machine.
 */

void dup(void)
{
  if (stack_p == MAX_STACK_DEPTH) {
    fprintf(stderr, "Stack overflow. Couldn't COPY\n");
    exit(1);
  } else {
    stack[stack_p] = stack[stack_p-1];
    stack_p++;
  }
}

/* A printing routine.
 */

void emit(void)
{
  if (stack_p == 0) {
    fprintf(stderr, "Stack underflow. Could't PRINT.\n");
    exit(1);
  } else
    switch (stack[--stack_p].type) {
    case NUMBER:
      printf("  %d\n", stack[stack_p].val.number);
      break;
    case SYMBOL:
      printf("  %s\n", stack[stack_p].val.symbol->name);
      break;
    default:
      fprintf(stderr, "Unknown data type. Contact maintainance :-)\n");
      break;
    }
}


void stack_dump(void)
{
  int tmp;

  if (stack_p != 0) {
    printf("STACK DUMP FOLLOWS\n");
    for (tmp = stack_p-1; tmp >= 0; tmp--) 
      if (stack[tmp].type == NUMBER)
	printf("\t%d\n", stack[tmp].val.number);
      else
	printf("\t%s\n", stack[tmp].val.symbol->name);
  }

  return;
}
