/* Simple symbol table entries with a binary tree.
 *
 * 2.10.89 Kenneth Oksanen
 *   First Version
 */

#include <stdio.h>
#include <string.h>

#include "defs.h"


static sym_tree_node_t *sym_tree_root = NULL;


static int mystrcmp(const char *const a, const char *const b)
{
  static int tmp;

  tmp = strcmp(a, b);
  if (tmp == 0) 
    return 0;
  else if (tmp < 0) 
    return -1;
  else
    return 1;
}
  

sym_tree_node_t *get_sym(const char *const name)
{
  sym_tree_node_t *walker = sym_tree_root;

  while (walker != NULL)
    switch (mystrcmp(walker->name, name)) {
    case -1:
      walker = walker->left;
      break;
    case 1:
      walker = walker->right;
      break;
    default:
      return walker;
      break;
    }

  return NULL;
}


static sym_tree_node_t *make_sym_node(void)
{
  sym_tree_node_t *res = (sym_tree_node_t *) malloc(sizeof(sym_tree_node_t));

  if (res == NULL) {
    fprintf(stderr, "Memory exhausted\n");
    exit(1);
  }
  return res;
}


sym_tree_node_t *add_sym(char *name)
{
  sym_tree_node_t *walker = sym_tree_root, *old;

  if (walker == NULL) {
    walker = make_sym_node();
    walker->left = walker->right = NULL;
    walker->data.address = -1;
    walker->name = name;
    sym_tree_root = walker;
    return sym_tree_root;
  }

  for (;;) {
    old = walker;
    switch (mystrcmp(walker->name, name)) {
    case -1:
      walker = walker->left;
      if (walker == NULL) {
	old->left = walker = make_sym_node();
	walker->left = walker->right = NULL;
	walker->data.address = -1;
	walker->name = name;
	return walker;
      }
      break;
    case 1:
      walker = walker->right;
      if (walker == NULL) {
	old->right = walker = make_sym_node();
	walker->left = walker->right = NULL;
	walker->data.address = -1;
	walker->name = name;
	return walker;
      }
      break;
    case 0:
      return walker;
    }
  }
}


/* Originally meant for debugging. I never needed it, but it's here if
 * you need it!
 */


static void do_print_syms(sym_tree_node_t *node)
{
  if (node == NULL) {
    printf("| ");
    return;
  }
  do_print_syms(node->left);
  printf("%s ", node->name);
  do_print_syms(node->right);
}


void print_syms(void)
{
  do_print_syms(sym_tree_root);
  printf("\n");
}




